@@ -17,6 +17,7 @@ from xpander_sdk import (
 from ...generic import get_sub_agent_id_from_oas_by_name, wait
 from .base import SDKAdapter
 import chainlit as cl
+from chainlit import make_async
 from chainlit.data.base import BaseDataLayer
 from chainlit.user import PersistedUser, User
 from chainlit.types import (
@@ -36,7 +37,6 @@ def utc_now():
 
 AGENT_END_TOOL = "xpfinish-agent-execution-finished"
 MAX_TOOL_CALLS = 15
-MAX_SAME_TOOL_IN_A_ROW = 5
 
 class ChainlitToolCallWithLLMResponse(BaseModel):
     tool_call: Any
@@ -197,12 +197,12 @@ class ChainlitXpanderDataLayer(BaseDataLayer):
         if not author:
             raise Exception("Thread author not found")
 
-        xpander = XpanderClient(
+        xpander = await make_async(XpanderClient)(
             api_key=getenv("AGENT_CONTROLLER_API_KEY", ""),
             base_url=getenv("AGENT_CONTROLLER_URL", ""),
             organization_id=author.get("organization_id"),
         )
-        Memory.delete_thread_by_id({"configuration": xpander.configuration}, thread_id=thread_id)
+        await make_async(Memory.delete_thread_by_id)({"configuration": xpander.configuration}, thread_id=thread_id)
         self.thread_authors_cache.pop(thread_id)
 
     async def list_threads(self, pagination: Pagination, filters: ThreadFilter) -> PaginatedResponse[ThreadDict]:
@@ -222,12 +222,12 @@ class ChainlitXpanderDataLayer(BaseDataLayer):
         
         organization_id = user.metadata['organization_id']
         agent_id = user.metadata['agent_id']
-        xpander = XpanderClient(
+        xpander = await make_async(XpanderClient)(
             api_key=getenv("AGENT_CONTROLLER_API_KEY", ""),
             base_url=getenv("AGENT_CONTROLLER_URL", ""),
             organization_id=organization_id,
         )
-        threads = Memory.fetch_user_threads(
+        threads = await make_async(Memory.fetch_user_threads)(
             agent={"configuration": xpander.configuration, "userDetails": {"id": filters.userId}}
         )
         
@@ -265,13 +265,13 @@ class ChainlitXpanderDataLayer(BaseDataLayer):
             ThreadDict: Thread details if found, else None.
         """
         user: User = cl.user_session.get("user")
-        xpander = XpanderClient(
+        xpander = await make_async(XpanderClient)(
             api_key=getenv("AGENT_CONTROLLER_API_KEY", ""),
             base_url=getenv("AGENT_CONTROLLER_URL", ""),
             organization_id=user.metadata.get("organization_id"),
         )
 
-        all_user_threads = Memory.fetch_user_threads(
+        all_user_threads = await make_async(Memory.fetch_user_threads)(
             agent={"configuration": xpander.configuration, "userDetails": {"id": user.identifier}}
         )
         thread_meta = next((trd for trd in all_user_threads if trd.id == thread_id), None)
@@ -311,12 +311,12 @@ class ChainlitXpanderDataLayer(BaseDataLayer):
         if not author or author == None:
             return
 
-        xpander = XpanderClient(
+        xpander = await make_async(XpanderClient)(
             api_key=getenv("AGENT_CONTROLLER_API_KEY", ""),
             base_url=getenv("AGENT_CONTROLLER_URL", ""),
             organization_id=author.get("organization_id"),
         )
-        Memory.rename_thread_by_id({"configuration": xpander.configuration}, thread_id=thread_id, name=name)
+        await make_async(Memory.rename_thread_by_id)({"configuration": xpander.configuration}, thread_id=thread_id, name=name)
 
     async def delete_user_session(self, id: str) -> bool:
         """
@@ -535,7 +535,6 @@ class ChainlitAdapter(SDKAdapter):
         current_execution = self.agent.execution.to_dict()
         user_settings = cl.user_session.get("user_settings",{})
         is_debug_mode_active = user_settings.get("debug_mode", True)
-        is_repeating_tool_call = False
         is_max_tool_call_reached = False
         
         is_sub_agent = True if "parentExecution" in current_execution and current_execution['parentExecution'] else False
@@ -555,13 +554,6 @@ class ChainlitAdapter(SDKAdapter):
             if len(self.execution_tool_calls[execution_id]) >= MAX_TOOL_CALLS:
                 is_max_tool_call_reached = True
                 
-            # Check if the last 3 tool calls are the same
-            if len(self.execution_tool_calls[execution_id]) >= MAX_SAME_TOOL_IN_A_ROW:
-                last_three_calls = self.execution_tool_calls[execution_id][-MAX_SAME_TOOL_IN_A_ROW:]
-                if len(set(last_three_calls)) == 1:  # All three are the same
-                    is_repeating_tool_call = True
-
-
         self.agent.add_messages(
             messages=[
                 {
@@ -697,11 +689,10 @@ class ChainlitAdapter(SDKAdapter):
                 self.agent.add_tool_call_results(tool_call_results=[tool_call_result])
             else:
                 # failures
-                if is_max_tool_call_reached or is_repeating_tool_call:
+                if is_max_tool_call_reached:
                     if is_max_tool_call_reached:
                         error = f"Oops! You've reached the maximum allowed tool executions ({MAX_TOOL_CALLS}). Please adjust your input or try again later."
-                    elif is_repeating_tool_call:
-                        error = f"Oops! The last {MAX_SAME_TOOL_IN_A_ROW} tool calls were the same ({last_three_calls[0]}). The agent cannot proceed with this tool."
+                
                     
                     tool_call_result = ToolCallResult(function_name=tool_call.name,tool_call_id=tool_call.tool_call_id,payload=tool_call.payload,result=error,is_success=False,is_error=True)
                     self.agent.add_tool_call_results(tool_call_results=[tool_call_result])
@@ -721,7 +712,7 @@ class ChainlitAdapter(SDKAdapter):
                         await current_step.update()
                     raise Exception(error)
                 else: # execute the tool
-                    tool_call_result = self.agent.run_tool(tool=tool_call, payload_extension={"headers":{"x-xpander-source-node-type": SourceNodeType.ASSISTANT.value.lower()}})
+                    tool_call_result = await make_async(self.agent.run_tool)(tool=tool_call, payload_extension={"headers":{"x-xpander-source-node-type": SourceNodeType.ASSISTANT.value.lower()}})
             
             current_step.end = utc_now()
             
